$.widget("wildix.navigation", {
    _create: function () {
        this._presenceEl = this.element.find(".navigation-presence");
        this._presenceAnchorEl = this._presenceEl.find("a");
        this._presenceStatusEl = this._presenceAnchorEl.find("span");
        this._presenceStatusIconEl = this._presenceAnchorEl.find("i");
        this._presenceAnchorEl.on("click", $.proxy(this._onPresenceClick, this));
        this._presencePopOverEl = this.element.find(".navigation-popover");
        this._statusSelect = this._presencePopOverEl.find("select.status");
        this._statusSelect.on("change", $.proxy(this._onPresenceStatusChange, this));
        this._untilInput = this._presencePopOverEl.find("input.until");
        this._messageInput = this._presencePopOverEl.find("input.message");
        this._applyButton = this._presencePopOverEl.find("button.apply");
        this._applyButton.on("click", $.proxy(this._onPresenceApplyButtonClick, this));
        this._cancelButton = this._presencePopOverEl.find("button.cancel");
        this._cancelButton.on("click", $.proxy(this._onPresenceCancelButtonClick, this));
    },

    // --------------------
    // Public API
    // --------------------

    changePersonalPresence: function(presence) {
        this._personal_presence = presence;

        this._presenceStatusIconEl.attr("class", "status");

        if (!presence.isOnline()) {
            // Disable views
            this._presenceStatusEl.html("Offline");
            this._presenceStatusIconEl.addClass("offline");
            return;
        }

        if (presence.isAway()) {
            this._presenceStatusEl.html("Away");
            this._presenceStatusIconEl.addClass("away");
            this._statusSelect.val("away");
            this._untilInput.show();
        } else if (presence.isDND()) {
            this._presenceStatusEl.html("DND");
            this._presenceStatusIconEl.addClass("dnd");
            this._statusSelect.val("dnd");
            this._untilInput.show();
        } else {
            this._presenceStatusEl.html("Online");
            this._presenceStatusIconEl.addClass("online");
            this._statusSelect.val("online");
            this._untilInput.hide();
        }

        if (presence.isStatusUntilAvailable()) {
            this._untilInput.val(this._formatDate(presence.getStatusUntil()));
        } else {
            var date = new Date();
            date.setTime(date.getTime() + (1*60*60*1000)); // Add two hours from now
            this._untilInput.val(this._formatDate(date));
        }

        this._messageInput.val(presence.getStatusMessage());
    },

    /**
     *
     */
    disconnected: function() {
        this._presenceEl.addClass("disabled");
        this._collapsePresencePopOver();
    },

    /**
     *
     */
    reconnected: function() {
        this._presenceEl.removeClass("disabled");
    },

    // --------------------
    // Presence functionality
    // --------------------

    _onPresenceClick: function() {
        if (this._presenceEl.hasClass("disabled")) {
            return;
        }

        if (this._presenceEl.hasClass("active")) {
            this._collapsePresencePopOver();
        } else {
            this._expandPresencePopOver();
        }
    },

    _onPresenceStatusChange: function() {
        var show = this._statusSelect.val();
        if (show == "away" || show == "dnd") {
            this._untilInput.show();
        } else {
            this._untilInput.hide();
        }
    },

    _onPresenceApplyButtonClick: function() {
        var builder = new WTAPI.Presence.Builder();

        // Set status
        switch(this._statusSelect.val()) {
            case "away":
                builder.setAway();
                break;
            case "dnd":
                builder.setDND();
                break;
        }

        // Set status message
        var message = this._messageInput.val();
        if (message.length > 0) {
            builder.setStatusMessage(message);
        }

        // Set status until
        var until = this._untilInput.val();
        if (until.length > 0) {
            builder.setStatusUntil(this._parseDate(until));
        }

        this._presenceEl.trigger("presence_changed", [builder.build()]);
        this._collapsePresencePopOver();
    },

    _onPresenceCancelButtonClick: function() {
        this._collapsePresencePopOver();
    },

    _collapsePresencePopOver: function() {
        this._presenceEl.removeClass("active");
        this._presencePopOverEl.hide();
    },

    _expandPresencePopOver: function() {
        this._presenceEl.addClass("active");
        this._presencePopOverEl.show();
    },

    /**
     * Format a date with a custom format.
     *
     * @private
     */
    _formatDate: function(date) {
        return date.getDate() + "/" + (date.getMonth() + 1) + "/" + date.getFullYear() + " " + date.getHours() + ":" + date.getMinutes();
    },

    /**
     * Parse a date with a custom format.
     *
     * @returns {Date|null}
     * @private
     */
    _parseDate: function(dateString) {
        var regex = /(\d{1,2})\/(\d{1,2})\/(\d{4})\s(\d{1,2}):(\d{1,2})/;
        var matches = regex.exec(dateString);
        if (matches === null) {
            return null;
        }

        var year = parseInt(matches[3]);
        var month = parseInt(matches[2]) - 1; // Careful, month starts at 0!
        var day = parseInt(matches[1]);
        var hours = parseInt(matches[4]);
        var minutes = parseInt(matches[5]);
        var seconds = 0;
        return new Date(year, month, day, hours, minutes, seconds);
    }
});