$.widget("wildix.potable", {
    _create: function () {
        this._rows = {};
        this._calls = this.options.calls || [];
        this._devices = this.options.devices || [];

        this._bodyElement = this.element.find("tbody");
        this._rowExampleRowElement = this.element.find("#po-table-template-row");
        this._emptyRowElemenet = this.element.find("#po-table-empty-row");
        this._notConnectedElemenet = this.element.find("#po-table-disconnected-row");
        this._notConnectedElemenet.hide();

        for (var i=0; i < this._calls.length; i++) {
            this._createRow(this._calls[i]);
        }

        setTimeout($.proxy(this._onDurationChanged, this), 1000);
    },

    enable: function() {
        for (var key in this._rows) {
            this._rows[key].porow("enable");
        }
    },

    disable: function() {
        for (var key in this._rows) {
            this._rows[key].porow("disable");
        }
    },

    onDevicesChanged: function (devices) {
        this._devices = devices;
        for (var key in this._rows) {
            var el = this._rows[key];
            $(el).porow("onDevicesChanged", devices);
        }
    },

    onCallAdded: function(call) {
        this._calls.push(call);
        this._createRow(call);

        this._hideEmptyRowsNotification();

        for (var key in this._rows) {
            var el = this._rows[key];
            $(el).porow("onCallsChanged", this._calls);
        }
    },

    onCallUpdated: function(call) {
        if (typeof(this._rows[call.getChannel()]) !== 'undefined') {
            var el = this._rows[call.getChannel()];
            $(el).porow("onCallUpdated", call);
        }

        for (var key in this._rows) {
            var el = this._rows[key];
            $(el).porow("onCallsChanged", this._calls);
        }
    },

    onCallTerminated: function(call) {
        if (typeof(this._rows[call.getChannel()]) !== 'undefined') {
            var el = this._rows[call.getChannel()];
            delete this._rows[call.getChannel()];
            $(el).remove();
        }

        var filtered = [];
        for (var i=0; i < this._calls.length; i++) {
            if (this._calls[i].getChannel() != call.getChannel()) {
                filtered.push(this._calls[i]);
            }
        }

        this._calls = filtered;

        if (this._calls.length == 0) {
            this._showEmptyRowsNotification();
        }

        for (var key in this._rows) {
            var el = this._rows[key];
            $(el).porow("onCallsChanged", this._calls);
        }
    },

    onConnected: function() {
        this._notConnectedElemenet.hide();
        this._emptyRowElemenet.show();
        this.enable();
        this.element.removeClass("disabled");
    },

    onDisconnected: function() {
        this._notConnectedElemenet.show();
        this._emptyRowElemenet.hide();
        this.disable();
        this.element.addClass("disabled");
    },

    _createRow: function(call) {
        var el = this._rowExampleRowElement.clone();
        el.attr("id", null);
        el.appendTo(this._bodyElement);
        this._rows[call.getChannel()] = el;

        var params = {
            call: call,
            activeCalls: this._calls,
            activeDevices: this._devices
        };
        $(el).porow(params);
    },

    _showEmptyRowsNotification: function() {
        $(this._emptyRowElemenet).show();
    },

    _hideEmptyRowsNotification: function() {
        $(this._emptyRowElemenet).hide();
    },

    _onDurationChanged: function() {
        for (var key in this._rows) {
            this._rows[key].porow("onDurationChanged");
        }

        setTimeout($.proxy(this._onDurationChanged, this), 1000);
    }
});