$.widget("wildix.queues", {
    _create: function () {
        this._columns = this.element.find("table td ul");

        this._modal = $("#queues-member-add-modal");
        this._modalTitle = this._modal.find('h3');
        this._modalInput = $("#queues-member-add-input");
        this._modalSubmit = $("#queues-member-add-button");
        this._modalSubmit.on('click', jQuery.proxy(this._onAddModalPressed, this));

        this._subscribeButton = this.element.find("button.subscribe");
        this._subscribeButton.on('click', jQuery.proxy(this._onSubscribePressed, this));
        this._unsubscribeButton = this.element.find("button.unsubscribe");
        this._unsubscribeButton.on('click', jQuery.proxy(this._onUnsubscribePressed, this));
        this._wrapper = this.element.find(".queues-wrapper");

        this._notSubscribedNotification = this.element.find('.not-subscribed');
        this._notSubscribedNotification.show();
        this._notConnectedNotification = this.element.find('.not-connected');

        $.template("queueMemberItem", '<tr class="member" rel="${location}">' +
            '<td class="member-name">${name}</td>' +
            '<td class="member-status">${status}</td>' +
            '<td class="member-calls"><a href="#">${calls}</a></td>' +
            '<td class="member-time">${time}</td>' +
            '<td class="member-actions">' +
                '<img class="pause" src="./assets/img/pause.png" width="16" height="16"/>' +
                '<img class="resume" src="./assets/img/resume.png" width="16" height="16"/>' +
                '<img class="remove" src="./assets/img/remove.png" width="16" height="16"/>' +
            '</td>' +
        '</tr>');

        $.template("queueUserItem", '<tr class="user" rel="${channel}">' +
            '<td class="user-name">${name}</td>' +
            '<td class="user-channel">${channel}</td>' +
            '<td class="user-duration">${duration}</td>' +
        '</tr>');

        $.template("queueItem", '<div class="queue" rel="${id}">' +
            '<div class="title">${name} <a class="label add" href="#">Add user</a></div>' +
            '<table class="users">' +
                '<thead>' +
                    '<tr>' +
                        '<th width="40%">Caller</th>' +
                        '<th width="35%">Channel</th>' +
                        '<th width="25%">Time</th>' +
                    '</tr>' +
                '</thead>' +
                '<tbody class="users" />' +
                '<tfoot><tr><td colspan="3" class="notification">No callers</td></tr></tfoot>' +
            '</table>' +
            '<table class="members">' +
                '<thead>' +
                    '<tr>' +
                        '<th width="40%">User</th>' +
                        '<th width="20%">Status</th>' +
                        '<th width="15%">Calls</th>' +
                        '<th width="10%">Talk</th>' +
                        '<th width="15%"></th>' +
                    '</tr>' +
                '</thead>' +
                '<tbody/>' +
                '<tfoot><tr><td colspan="5" class="notification">No users</td></tr></tfoot>' +
            '</table>' +
        '</div>');
    },

    // --------------------
    // Public API
    // --------------------

    reconnected: function() {
        this.enableActions();
        this._notConnectedNotification.hide();

        if (this._subscribeButton.is(':visible')) {
            this._notSubscribedNotification.show();
        }
    },

    disconnected: function() {
        console.log('disconnected');
        this.switchToDisconnectedState();
    },

    disableActions: function() {
        this._subscribeButton.attr('disabled', true);
        this._unsubscribeButton.attr('disabled', true);
    },

    enableActions: function() {
        this._subscribeButton.attr('disabled', false);
        this._unsubscribeButton.attr('disabled', false);
    },

    switchToSubscribedState: function() {
        this._subscribeButton.hide();
        this._unsubscribeButton.show();

        this._notSubscribedNotification.hide();
        this._notConnectedNotification.hide();
    },

    switchToUnsubscribedState: function() {
        this._subscribeButton.show();
        this._unsubscribeButton.hide();
        this._notSubscribedNotification.show();
        this._notConnectedNotification.hide();
    },

    switchToDisconnectedState: function() {
        this.disableActions();
        this._notSubscribedNotification.hide();
        this._notConnectedNotification.show();
    },

    addQueue: function(queue) {
        var el = $.tmpl("queueItem", {id: queue.getId(), name: queue.getName()}).appendTo(this._wrapper);
        el.find('.title .add').on('click', jQuery.proxy(this._onAddPressed, this, queue));

        this.drawQueue(queue);
    },

    drawQueue: function (queue) {
        var el = $('.queue[rel="'+queue.getId()+'"]');
        el.find('.members tbody').empty();
        el.find('.users tbody').empty();
        var i;
        // Process members
        var members = queue.getMembers();
        members = members.sort(function (a, b) {
            return a._extension - b._extension ;
        });

        var membersCnt = el.find('.members tbody');
        for (i = 0; i < members.length; i++) {
            this.addMember(queue, members[i], membersCnt);
        }

        // Process users
        var users = queue.getUsers();
        var usersCnt = el.find('.users tbody');
        for (i = 0; i < users.length; i++) {
            this.addUser(queue, users[i], usersCnt);
        }
    },

    removeQueue: function(queue) {
        this._getQueueEl(queue).remove();
    },

    addMember: function(queue, member, container) {
        var cnt = container || this._getQueueMembersContainer(queue);
        var type = member.isDynamicMember() ? 'dynamic' : 'static';
        var name = member.getExtension() + ' ('+ type +' / '+ member.getPenalty() +' penalty)';
        var time = this._formatMemberTime(member);
        var params = {
            name: name,
            location: member.getLocation(),
            status: member.getStatus(),
            calls: member.getCalls().length,
            time: time
        };
        var el = $.tmpl("queueMemberItem", params).appendTo(cnt);

        // ...
        var pauseEl = el.find('img.pause');
        var resumeEl = el.find('img.resume');
        var removeEl = el.find('img.remove');
        pauseEl.on('click', jQuery.proxy(this._onPausePressed, this, queue, member));
        pauseEl.tooltip({title: "Pause Member", placement: "right", html: true});
        resumeEl.on('click', jQuery.proxy(this._onResumePressed, this, queue, member));
        resumeEl.tooltip({title: "Resume Member", placement: "right", html: true});
        removeEl.on('click', jQuery.proxy(this._onRemovePressed, this, queue, member));
        removeEl.tooltip({title: "Remove Member from Queue", placement: "right", html: true});
        $('#queues-member-add-input').on('keypress', function () {
            $(this).css('border','none');
        });

        if (member.isPaused()) {
            pauseEl.hide();
            resumeEl.show();
        } else {
            pauseEl.show();
            resumeEl.hide();
        }

        if (member.isDynamicMember()) {
            removeEl.show();
        } else {
            removeEl.hide();
        }

        // ...
        var callsEl = el.find('.member-calls a');
        callsEl.tooltip({title: this._formatCallsDescription(member), placement: "right", html: true});

        // Hide notification
        if (queue.getMembers().length > 0) {
            this._getQueueEl(queue).find('.members tfoot').hide();
        }
    },

    updateMember: function(queue, member) {
        var el = this._getMemberEl(queue, member);

        // Update data
        el.find('.member-status').html(member.getStatus());
        el.find('.member-calls a').html(member.getCalls().length);

        // Update actions
        var pauseEl = el.find('img.pause');
        var resumeEl = el.find('img.resume');
        var removeEl = el.find('img.remove');
        if (member.isPaused()) {
            pauseEl.hide();
            resumeEl.show();
        } else {
            pauseEl.show();
            resumeEl.hide();
        }

        if (member.isDynamicMember()) {
            removeEl.show();
        } else {
            removeEl.hide();
        }
    },

    updateMemberCalls: function(queue, member) {
        var el = this._getMemberEl(queue, member);
        el.find('.member-calls a').html(member.getCalls().length);

        // ...
        var callsEl = el.find('.member-calls a');
        callsEl.tooltip("destroy");
        callsEl.tooltip({title: this._formatCallsDescription(member), placement: "right", html: true});
    },

    updateMemberTime: function(queue, member) {
        var el = this._getMemberEl(queue, member);
        var time = this._formatMemberTime(member);

        el.find('.member-time').html(time);
    },

    removeMember: function(queue, member) {
        this._getMemberEl(queue, member).remove();

        // Show notification
        if (queue.getMembers().length == 0) {
            this._getQueueEl(queue).find('.members tfoot').show();
        }
    },

    addUser: function(queue, user, container) {
        var cnt = container || this._getQueueUsersContainer(queue);
        // var channel = user.getChannel().length > 20 ? user.getChannel().subscr :
        var params = {
            channel: user.getChannel(),
            name: this._formatCallerName(user),
            duration: user.getDuration()
        };

        if (!cnt) {
            return;
        }

        $.tmpl("queueUserItem", params).appendTo(cnt);

        // Hide notification
        if (queue.getUsers().length > 0) {
            this._getQueueEl(queue).find('.users tfoot').hide();
        }
    },

    removeUser: function(queue, user) {
        this._getUserEl(queue, user).remove();

        // Show notification
        if (queue.getUsers().length == 0) {
            this._getQueueEl(queue).find('.users tfoot').show();
        }
    },

    updateUserDuration: function(queue, user) {
        var el = this._getUserEl(queue, user);
        el.find('.user-duration').html(user.getDuration());
    },

    enable: function() {
        this.element.removeClass('disabled');
    },

    disable: function() {
        this.element.addClass('disabled');
    },

    // --------------------
    // Private functionality
    // --------------------

    _onSubscribePressed: function() {
        this.element.trigger("subscribe");
    },

    _onUnsubscribePressed: function() {
        this.element.trigger("unsubscribe");
    },

    _onAddPressed: function(queue) {
        this._modalTitle.html("Call Group #" + queue.getId());
        this._modalInput.val("");
        this._modalQueue = queue;
        this._modal.modal('show');
    },

    _onAddModalPressed: function() {
        var extension = this._modalInput.val();
        var queue = this._modalQueue;

        if(!extension) {
            $('#queues-member-add-input').css('border', '2px solid #FDADAF');
            return false;
        }

        this.element.trigger("member_add", [queue, extension]);
        this._modal.modal('hide');

        return false;
    },

    _onPausePressed: function(queue, member) {
        this.element.trigger("member_pause", [queue, member]);
    },

    _onResumePressed: function(queue, member) {
        this.element.trigger("member_resume", [queue, member]);
    },

    _onRemovePressed: function(queue, member) {
        this.element.trigger("member_remove", [queue, member]);
    },

    _getQueueEl: function(queue) {
        var el = this.element.find('.queue[rel="'+ queue.getId() +'"]');
        return el;
    },

    _getQueueMembersContainer: function(queue) {
        return this._getQueueEl(queue).find('.members tbody');
    },

    _getQueueUsersContainer: function(queue) {
        return this._getQueueEl(queue).find('.users tbody');
    },

    _getMemberEl: function(queue, member) {
        var cnt = this._getQueueEl(queue);
        var el = cnt.find('.member[rel="'+ member.getLocation() +'"]');

        return el;
    },

    _getUserEl: function(queue, user) {
        var cnt = this._getQueueEl(queue);
        var el = cnt.find('.user[rel="'+ user.getChannel() +'"]');

        return el;
    },

    _formatCallsDescription: function(member) {
        var calls = member.getCalls();
        if (calls.length == 0) {
            return "No calls";
        }

        var description = "<div>";
        for (var i=0; i < calls.length; i++) {
            var call = calls[i];

            if (call.getCalleeNumber() == call.getCalleeName() || call.getCalleeName() == "") {
                description += "<div>"+ call.getCalleeNumber() +"</div>";
            } else {
                description += "<div>"+ call.getCalleeName() +" ("+ call.getCalleeNumber() +")</div>";
            }
        }
        description += "</div>";
        return description;
    },


    _formatCallerName: function(user) {
        if (user.getName() == "" || user.getNumber() == user.getName()) {
            return user.getNumber();
        }
        return user.getName() + " (" + user.getNumber() + ")";
    },

    _formatMemberTime: function(member) {
        var time = member.getCalls() == 0 ? "-" + member.getSleepTime() : member.getTalkTime();
        return time;
    }
});