$.widget("wildix.queue", {
    _create: function () {
        this.queue = this.options.queue;

        this.sla_wait = this.options.sla_wait;
        this.sla_percentage = this.options.sla_percentage;
        this.sla_success = 0; // Calls count that were answered less than XX (sla_timeout) seconds
        this.sla_failure = 0; // Calls count that were answered more than XX (sla_timeout) seconds

        this.calls_total = 0;
        this.calls_wait = 0;
        this.call_unanswered = {};

        this.members = {};

        $.template("queueMemberItem", '<tr class="call" rel="${location}">' +
            '<td class="center presence"><div></div></td>' +
            '<td class="center calls"><a href="#">-</a></td>' +
            '<td class="name">${name}</td>' +
            '<td class="center global-talk-time">-</td>' +
            '<td class="center global-pause-time">-</td>' +
            '<td class="center pause-time">-</td>' +
            '<td class="center callee">-</td>' +
            '<td class="center callee-time">-</td>' +
        '</tr>');

        $.template("queueUserItem", '<tr class="customer" rel="${channel}">' +
            '<td colspan="6">&nbsp;</td>' +
            '<td class="center callee">${name}</td>' +
            '<td class="center duration">${duration}</td>' +
        '</tr>');

        $.template("queue", '<table>' +
            '<thead>' +
                '<tr class="header">' +
                    '<td colspan="9">' +
                        '<div class="stats"><a href="#">SLA: <b class="sla-percentage success">100%</b> and medium wait time is <b class="sla-wait success">0</b></a> (${sla_wait} seconds for ${sla_percentage}% of calls)</div>' +
                        '<div class="title">${name}</div>' +
                    '</td>' +
                '</tr>' +
                '<tr class="info">' +
                    '<td colspan="2">Agents</td>' +
                    '<td><span class="agents-count">-</span></td>' +
                    '<td colspan="2">Pause</td>' +
                    '<td><span class="agents-pause">-</span></td>' +
                    '<td>Calls on queue</td>' +
                    '<td><span class="calls-active">-</span></td>' +
                '</tr>' +
                    '<tr class="info">' +
                    '<td colspan="2">On call</td>' +
                    '<td><span class="agents-talking">-</span></td>' +
                    '<td colspan="2">Free</td>' +
                    '<td><span class="agents-free">-</span></td>' +
                    '<td>Total calls</td>' +
                    '<td><span class="calls-total">-</span></td>' +
                '</tr>' +
                '<tr class="info-members">' +
                    '<td style="width: 21px;"><div class="add"></div></td>' +
                    '<td style="width: 21px;">Calls</td>' +
                    '<td style="width: 125px;">Agent</td>' +
                    '<td style="width: 41px;">Global Call Time</td>' +
                    '<td style="width: 41px;">Global Pause Time</td>' +
                    '<td style="width: 41px;">Pause<br/>Time</td>' +
                    '<td style="width: 120px;">Customer</td>' +
                    '<td style="width: 93px;">Talking time<br/>(Waiting time)</td>' +
                '</tr>' +
            '</thead>' +
            '<tbody></tbody>' +
            '<tfoot></tfoot>' +
        '</table>');


        var params = {
            id: this.queue.getId(),
            name: this.queue.getName(),
            sla_wait: this.sla_wait,
            sla_percentage: this.sla_percentage
        };
        $.tmpl("queue", params).appendTo(this.element);

        this.element.find(".stats a").on('click', jQuery.proxy(this.onResetSLA, this));
        this.element.find(".add").on('click', jQuery.proxy(this.onAddMember, this));

        this.drawTable();

        this.element.css("min-height", ($(window).height() - 92) + "px");
    },

    drawTable: function () {
        // ...
        var queue = this.queue;
        this.element.find('tbody').empty();
        this.element.find('tfoot').empty();

        // Process members
        var members = queue.getMembers();

        members = members.sort(function (a, b) {
            return a._name.localeCompare( b._name );
        });

        var membersCnt = this.element.find('tbody');
        var i;
        for (i = 0; i < members.length; i++) {
            this.addMember(members[i], membersCnt);
        }

        // Process users
        var users = queue.getUsers();
        var usersCnt = this.element.find('tfoot');
        for (i = 0; i < users.length; i++) {
            this.addUser(users[i], usersCnt);
        }


        this.updateSummary();
    },

    // --------------------
    // Public API
    // --------------------

    /**
     * Method executes each second
     */
    time: function() {
        var members = this.queue.getMembers();
        var el;
        var i;
        for (i = 0; i < members.length; i++) {
            var member = members[i];
            el = this._getMemberEl(member);

            this.updateMemberCallee(member, el);
            this.updateMemberTimers(member, el);
        }

        var users = this.queue.getUsers();
        for (i = 0; i < users.length; i++) {
            var user = users[i];
            el = this._getUserEl(user);

            this.updateUserTimers(user, el);
        }
    },

    addMember: function(member, container) {
        var cnt = container || this.element.find('tbody');
        var name = member.getName() + " (" + member.getExtension() + ")";
        var time = this._formatMemberTime(member);
        var params = {
            name: name,
            location: member.getLocation(),
            status: member.getStatus(),
            calls: member.getCalls().length,
            time: time
        };
        var el = $.tmpl("queueMemberItem", params).appendTo(cnt);
        el.find('.calls a').on('click', jQuery.proxy(this.onResetMember, this, member));

        this.members[member.getLocation()] = {
            'calls': 0,
            'talk_time': 0,
            'talk_time_initial': member.getTalkTime(),
            'pause_time': 0,
            'pause_time_initial': member.getSleepTime()
        };

        this.updateMember(member, el);
        this.updateMemberCallee(member, el);
        this.updateMemberCalls(member, el);
        this.updateMemberTimers(member, el);
        this.updateSummary();

        // Update presence info
        var user = wtapi.roster.getUserByExtension(member.getExtension());
        if (user == null) {
            user = wtapi.roster.createUserByExtension(member.getExtension());
        }
        var presence = wtapi.presence.get(user);
        this.updateMemberPresence(member, presence);
    },

    updateMember: function(member, element) {
        var el = element || this._getMemberEl(member);
        this.updateMemberCallee(member, el);

        var user = wtapi.roster.getUserByExtension(member.getExtension());
        if (user == null) {
            user = wtapi.roster.createUserByExtension(member.getExtension());
        }
        var presence = wtapi.presence.get(user);
        this.updateMemberPresence(member, presence);
    },

    updateMemberCallee: function(member, element) {
        var el = element || this._getMemberEl(member);

        // Update callee
        if (member.getCalls().length > 0) {
            var call = member.getCalls()[0];
            var id = call.getChannel() + call.getCalleeNumber();
            var wait = this.call_unanswered[id] ? this.call_unanswered[id] : 0;

            el.find('.callee').text(this._formatName(call.getCalleeName(), call.getCalleeNumber()));
            el.find('.callee-time').html(this._formatDuration(call.getDuration()) + "<br />(" + this._formatDuration(wait) + ")");


        } else {
            el.find('.callee').text("-");
            el.find('.callee-time').text("-");
        }
    },

    updateMemberTimers: function(member, element) {
        var el = element || this._getMemberEl(member);
        var talk_time = parseInt(this.members[member.getLocation()]['talk_time']) - this.members[member.getLocation()]['talk_time_initial'];
        var pause_time = parseInt(this.members[member.getLocation()]['pause_time']) - this.members[member.getLocation()]['pause_time_initial'];

        if (member.getCalls().length == 0) {
            pause_time += member.getSleepTime();
        } else {
            talk_time += member.getCalls()[0].getDuration();
        }

        el.find('.global-talk-time').text(this._formatDuration(talk_time));
        el.find('.global-pause-time').text(this._formatDuration(pause_time));
        el.find('.pause-time').text(member.getCalls().length == 0 ? this._formatDuration(member.getSleepTime()) : "-" );
    },

    updateMemberCalls: function(member, element, count) {
        var el = element || this._getMemberEl(member);
        count = this.members[member.getLocation()]['calls'];

        // Update total calls
        if (count > 0) {
            el.find('.calls a').text(count);
        } else {
            el.find('.calls a').text(0);
        }
    },

    updateMemberTime: function(member) {
        var el = this._getMemberEl(member);
        var time = this._formatMemberTime(member);

        el.find('.member-time').html(time);
    },

    updateMemberPresence: function(member, presence) {
        var el = this._getMemberEl(member);
        var presenceEl = el.find('.presence');
        presenceEl.attr('class', 'center presence');

        if (member.isPaused()) {
            presenceEl.addClass("pause");
        } else if (presence.isTalking()) {
            presenceEl.addClass("talking");
        } else if (presence.isRinging()) {
            presenceEl.addClass("ringing");
        } else if (presence.isDND()) {
            presenceEl.addClass("dnd");
        } else if (presence.isAway()) {
            presenceEl.addClass("away");
        } else if (!presence.isOnline()) {
            presenceEl.addClass("offline");
        }
    },

    updateSummary: function() {
        var members = this.queue.getMembers();
        var paused = 0;
        var talking = 0;
        var free = 0;
        var calls = this.queue.getUsers().length;

        for (var i=0; i < members.length; i++) {
            var member = members[i];
            if (member.isPaused()) {
                paused++;
            } else if (member.getCalls().length > 0) {
                talking++;
            } else {
                free++;
            }

            calls += member.getCalls().length;
        }

        this.element.find("thead .agents-count").text(members.length);
        this.element.find("thead .agents-pause").text(paused);
        this.element.find("thead .agents-talking").text(talking);
        this.element.find("thead .agents-free").text(free);
        this.element.find("thead .calls-active").text(calls);
        this.element.find("thead .calls-total").text(this.calls_total);
    },
    
    updateSLA: function() {
        if (this.sla_success > 0 || this.sla_failure > 0) {
            var percentage = this.sla_failure > 0 ? 100 - Math.round(this.sla_failure / this.calls_total * 100) : 100;
            var percentageEl = this.element.find("thead .sla-percentage");
            var wait = this.calls_total > 0 ? Math.round(this.calls_wait / this.calls_total) : this.calls_wait;
            var waitEl = this.element.find("thead .sla-wait");

            // -----
            if (percentage >= this.sla_percentage) {
                percentageEl.addClass("success"); // Goal is reached
            } else {
                percentageEl.removeClass("success"); // Goal is failed
            }
            percentageEl.text(percentage + "%");

            // -----
            if (wait <= this.sla_wait) {
                waitEl.addClass("success"); // Goal is reached
            } else {
                waitEl.removeClass("success"); // Goal is failed
            }
            waitEl.text(wait);
        } else {
            this.element.find("thead .sla-percentage").text("100%").addClass("success");
            this.element.find("thead .sla-wait").text("0").addClass("success");
        }
    },

    removeMember: function(member) {
        this._getMemberEl(member).remove();
    },


    addCall: function(member, call) {
        // Increase SLA
        var id = call.getChannel() + call.getCalleeNumber();
        var success = true;
        if (this.call_unanswered[id]) {
            if (this.call_unanswered[id] > this.sla_wait) {
                success = false;
            }
        }

        if (success) {
            this.sla_success++;
        }

        // Increase Member calls
        this.members[member.getLocation()]['calls']++;
        this.members[member.getLocation()]['pause_time'] += member.getSleepTime();

        // Update element
        var el = this._getMemberEl(member);
        this.updateMemberCallee(member, el);
        this.updateMemberCalls(member, el);
        this.updateSummary();
        this.updateSLA();
    },

    removeCall: function(member, call) {
        this.members[member.getLocation()]['talk_time'] += call.getDuration();

        // -----
        var id = call.getChannel() + call.getCalleeNumber();
        delete this.call_unanswered[id];

        // -----
        var el = this._getMemberEl(member);
        this.updateMemberCallee(member, el);
        this.updateMemberTimers(member, el);
        this.updateSummary();
        this.updateSLA();
    },



    addUser: function(user, container) {
        // Increase statistics
        this.calls_total++;

        // Render element
        var cnt = container || this.element.find('tfoot');
        var params = {
            channel: user.getChannel(),
            name: this._formatName(user.getName(), user.getNumber()),
            duration: this._formatDuration(user.getDuration())
        };

        if (!cnt) {
            return;
        }

        $.tmpl("queueUserItem", params).appendTo(cnt);

        this.updateSummary();
    },

    updateUserTimers: function(user, el) {
        el = el || this._getUserEl(user);

        if (user.getDuration() > this.sla_wait) {
            el.addClass("timeout");
        }

        el.find('.duration').text(this._formatDuration(user.getDuration()));
    },

    removeUser: function(user) {
        this._getUserEl(user).remove();

        // Save duration to
        var id = user.getChannel() + user.getNumber();
        this.call_unanswered[id] = user.getDuration();
        this.calls_wait += user.getDuration();

        if (user.getDuration() > this.sla_wait) {
            this.sla_failure++;
            this.updateSLA();
        }

        this.updateSummary();
    },


    setPresence: function(extension, presence) {
        var members = this.queue.getMembers();
        for (var i=0; i < members.length; i++) {
            var member = members[i];
            if (member.getExtension() == extension) {
                this.updateMemberPresence(member, presence);
                break;
            }
        }
    },

    // --------------------
    // Private functionality
    // --------------------

    _getMemberEl: function(member) {
        return this.element.find('.call[rel="'+ member.getLocation() +'"]');
    },

    _getUserEl: function(user) {
        return this.element.find('.customer[rel="'+ user.getChannel() +'"]');
    },

    _formatCallsDescription: function(member) {
        var calls = member.getCalls();
        if (calls.length == 0) {
            return "No calls";
        }

        var description = "<div>";
        for (var i=0; i < calls.length; i++) {
            var call = calls[i];

            if (call.getCalleeNumber() == call.getCalleeName() || call.getCalleeName() == "") {
                description += "<div>"+ call.getCalleeNumber() +"</div>";
            } else {
                description += "<div>"+ call.getCalleeName() +" ("+ call.getCalleeNumber() +")</div>";
            }
        }
        description += "</div>";
        return description;
    },


    _formatCallerName: function(user) {
        if (user.getName() == "" || user.getNumber() == user.getName()) {
            return user.getNumber();
        }
        return user.getName() + " (" + user.getNumber() + ")";
    },

    _formatName: function(name, number) {
        if (name == "" || number == name) {
            return number;
        }
        return name + " (" + number + ")";
    },

    _formatDuration: function(seconds) {
        if (isNaN(seconds)) {
            return "0:00:00";
        }
        var hours = parseInt( seconds / 3600 ) % 24;
        var minutes = parseInt( seconds / 60 ) % 60;
        seconds = seconds % 60;

        return hours + ":" + (minutes < 10 ? "0" + minutes : minutes) + ":" + (seconds  < 10 ? "0" + seconds : seconds);

        /*if (name == "" || number == name) {
            return number;
        }
        return name + " (" + number + ")";*/
    },

    _formatMemberTime: function(member) {
        return member.getCalls() == 0 ? "-" + member.getSleepTime() : member.getTalkTime();
    },

    onAddMember: function() {
        var extension = prompt("Please enter extension");
        if (extension && /^\d+$/.test(extension)) {
            this.element.trigger("add", [extension]);
        }
    },

    onResetMember: function(member) {
        if (confirm('Reset statistics for ' + member.getName() + " ?")) {
            this.members[member.getLocation()] = {
                'calls': 0,
                'talk_time': 0,
                'talk_time_initial': member.getTalkTime(),
                'pause_time': 0,
                'pause_time_initial': member.getSleepTime()
            };

            this.updateMemberCalls(member);
            this.updateMemberTimers(member);
        }
    },

    onResetSLA: function() {
        if (confirm('Reset SLA ?')) {
            this.sla_success = 0;
            this.sla_failure = 0;
            this.calls_total = 0;
            this.calls_wait = 0;
            this.call_unanswered = {};

            this.updateSLA();
        }
    }
});